@@ -124,8 +124,14 @@ log_db_error(const char *msg, int error)
 	syslog(LOG_ERR, "%s: %s", msg, db_strerror(error));
 }
 
+#if DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR < 3 || DB_VERSION_MAJOR < 4
+/* http://www.oracle.com/technology/documentation/berkeley-db/db/ref/upgrade.4.3/err.html */
+static void
+db_errcall_fcn(const char *errpfx, char *msg)
+#else
 static void
 db_errcall_fcn(const DB_ENV *dbenv, const char *errpfx, const char *msg)
+#endif
 {
     syslog(LOG_ERR, "%s: %s", errpfx ? errpfx : "Berkeley DB", msg);
 }
@@ -575,6 +581,91 @@ optp(const char *opt, const char *short_opt, const char *long_opt)
     return strcmp(opt, short_opt) == 0 || loptp(opt, long_opt);
 }
 
+static void
+print_usage(FILE *f, const char *progname)
+{
+    fprintf(f,
+	    "Usage: %s [-V] [-v] [-d] [-h <Berkeley DB home directory>] [-g <greylist delay>]\n"
+	    "    [-b <bloc maximum idle>] [-p <pass maximum idle>] [-r <reject action>]\n"
+	    "    [-G <greylisted action>] [-/ <network bits>] [--dump-triplets] [--help]\n"
+	    "\n"
+	    "    -b <seconds>, --bloc-max-idle <seconds>\n"
+	    "\n"
+	    "	This determines how many seconds of life are given to a record\n"
+	    "	that is created from a new mail (ip, from, to) triplet.  Note\n"
+	    "	that the window created by this setting for passing mails is\n"
+	    "	reduced by the amount set for --greylist-delay.  NOTE: See\n"
+	    "	also --pass-max-idle.  Defaults to %d.\n"
+	    "\n"
+	    "    -d, --debug\n"
+	    "\n"
+	    "	Debug logging\n"
+	    "\n"
+	    "    -g <seconds>, --greylist-delay <seconds>\n"
+	    "\n"
+	    "	This determines how many seconds we will block inbound mail\n"
+	    "	that is from a previously unknown (ip, from, to) triplet.  If\n"
+	    "	it is set to zero, incoming mail association will be learned,\n"
+	    "	but no deliveries will be tempfailed.  Use a setting of zero\n"
+	    "	with caution, as it will learn spammers as well as legitimate\n"
+	    "	senders.  Defaults to %d.\n"
+	    "\n"
+	    "    -h <Berkeley DB home directory>, --home <Berkeley DB home directory>\n"
+	    "\n"
+	    "	Location of the Berkeley DB environment home location (the\n"
+	    "	default is " DEF_DB_HOME ".\n"
+	    "\n"
+	    "    --help\n"
+	    "\n"
+	    "        Show usage information.\n"
+	    "\n"
+	    "    -p <seconds>, --pass-max-idle <seconds>\n"
+	    "\n"
+	    "	How much life (in secs) to give to a record we are updating\n"
+	    "	from an allowed (passed) email.\n"
+	    "\n"
+	    "	The default is %d, which should be enough to handle\n"
+	    "	messages that may only be sent once a month, or on things like\n"
+	    "	the first monday of the month (which sometimes means 5 weeks).\n"
+	    "	Plus, we add a day for a delivery buffer.\n"
+	    "\n"
+	    "    -r <reject action>, --reject-action <reject action>\n"
+	    "\n"
+	    "        The reject action directive that will be used.  See access(5)\n"
+	    "        for valid actions.  The string expands %%d to the number of\n"
+	    "        seconds, %%p to the empty string if %%d expands to 1 or \"s\"\n"
+	    "        otherwise, %%s to \" \" and %%%% to \"%%\".\n"
+	    "\n"
+	    "        The default is \"%s\".\n"
+	    "\n"
+	    "    -G <greylisted action>, --greylisted-action <greylisted action>\n"
+	    "\n"
+	    "        The action that will be used the first time a triplet passes\n"
+	    "        greylisting.  Same expansion as for --reject-action.\n"
+	    "\n"
+	    "        The default is \"%s\"\n"
+	    "\n"
+	    "    -v, --verbose\n"
+	    "\n"
+	    "	Verbose logging\n"
+	    "\n"
+	    "    -V, --version\n"
+	    "\n"
+	    "        Show version information.\n"
+	    "\n"
+	    "    -/ <nbits>, --network-prefix <nbits>\n"
+	    "\n"
+	    "	Only consider the first <nbits> bits of an IPv4 address.\n"
+	    "	Defaults to 32 i.e. the whole adresse is significant.\n"
+	    "\n"
+	    "    --dump-triplets\n"
+	    "\n"
+	    "        Dump the triplets database to stdout.  Mostly for debugging\n"
+	    "        purposes.\n",
+	    progname, AUTO_RECORD_LIFE_SECS, DELAY_MAIL_SECS,
+	    UPDATE_RECORD_LIFE_SECS, reject_action_fmt, greylisted_action_fmt);
+}
+
 int
 main(int argc, const char **argv)
 {
@@ -583,6 +674,7 @@ main(int argc, const char **argv)
     int rc;
     unsigned long network_prefix = 32;
     unsigned int suffix;
+    int opt_version = 0, opt_help = 0, opt_error = 0;
     progname = strrchr(argv[0], '/');
     if (progname)
 	progname++;
@@ -594,6 +686,10 @@ main(int argc, const char **argv)
 	    debug_me = 1;
 	else if (optp(argv[i], "-v", "--verbose"))
 	    opt_verbose++;
+	else if (optp(argv[i], "-V", "--version"))
+	    opt_version = 1;
+	else if (loptp(argv[i], "--help"))
+	    opt_help = 1;
 	else if (optp(argv[i], "-b", "--bloc-max-idle")) {
 	    if (++i >= argc)
 		fatal("Missing argument to --bloc-max-idle");
@@ -646,9 +742,20 @@ main(int argc, const char **argv)
 	}
 	else {
 	    fprintf(stderr, "Unknown option \"%s\"\n", argv[i]);
-	    exit(EXIT_FAILURE);
+	    opt_error = 1;
 	}
     }
+    if (opt_error) {
+	print_usage(stderr, progname);
+	exit(EXIT_FAILURE);
+    }
+    if (opt_version)
+	puts(PACKAGE_STRING);
+    if (opt_help) {
+	print_usage(stdout, progname);
+	exit(EXIT_SUCCESS);
+    }
+
     suffix = 32 - network_prefix;
     ipv4_network_strip_bytes = suffix >> 3;
     ipv4_network_mask = 0xffU & ~0U << (suffix & 0x7U);
